@@ -9,19 +9,21 @@
 #' @param lagmat A \eqn{k} by \eqn{2} matrix of spatial lags. Each row corresponds to a lag of the form \eqn{(x.lag, y.lag)} for which the semivariogram value will be estimated.
 #' @param A	A \eqn{d} by \eqn{k} contrast matrix. The contrasts correspond to contrasts of the estimated semivariogram at the lags given in 'lagmat'.
 #' @param df A scalar indicating the row rank of A. This value gives the degrees of freedom for the asymptotic Chi-sq distribution used to compute the p-value.
-#' @param subblock.dims	A vector of length two corresponding to the width and height of the moving windows used to estimate the asymptotic variance-covariance matrix. If window width does not evenly divide the width of the sampling region, some data will be ommited during subsampling, i.e., function does not handle partial windows. Same applies to window height and height of sampling region.
 #' @param h A scalar giving the bandwidth for the kernel smoother. The same bandwidth is used for lags in both the x and y directions.
 #' @param kernel A string taking one of the following values: "norm", "ep", "cos", or "unif", for the normal, Epanechnikov, cosine, or uniform kernel functions. Defaults to normal.
 #' @param truncation A scalar providing the truncation value for the normal density if 'kernel' is given as "norm".
 #' @param xlims A vector of length two providing the lower and upper x-limits of the sampling region.
 #' @param ylims A vector of length two providing the lower and upper y-limits of the sampling region.
-#' @param grid.spacing A scalar indicating the spacing of the underlying grid laid on the sampling region to create moving windows. The same spacing is used in the x and y directions.
+#' @param grid.spacing A vector of length two providing the x (width) and y (height) spacing, respectively, of the underlying grid laid on the sampling region to create moving windows. If the grid spacing width does not evenly divide the width of the sampling region, some data will be ommited during subsampling, i.e., the function does not handle partial windows. Same applies to grid spacing height and height of sampling region. See details for an example.
+#' @param window.dims A vector of length two corresponding to the width and height of the moving windows used to estimate the asymptotic variance-covariance matrix. The dimensions are given in terms of the spacing of the grid laid on the sampling region. See details for an example.
 #' @param subblock.h A scalar giving the bandwidth used for the kernel smoother when estimating the semivariogram on the moving windows (blocks). It is recommended to be less than 1 to maintain nominal test size.
 #' @param sig.est.finite Logical. True provides a finite sample correction in estimating Sigma (see Guan et. al. (2004) Section 4.2.2). False provides the empirical variance-covariance matrix of sample semivariogram values computed via the moving windows.
 #'
-#' @details This function currently only supports square and rectangular sampling regions and does not currently support partial blocks. For example, suppose the sampling region runs from 0 to 20 in the x-direction and from 0 to 30 in the y-direction and an underlying grid of 1 by 1 is laid over the sampling region. Then an ideal value of subblock.dims would be (2,3) since its entries evenly divide the width (20) and height (30), respectively, of the sampling region. Using the vector (3, 4.5) would imply that some data will not be used in the moving windows since these values would create partial blocks in the sampling region. 
+#' @details This function currently only supports square and rectangular sampling regions and does not support partial blocks. For example, suppose the sampling region runs from 0 to 20 in the x-direction and from 0 to 30 in the y-direction and an underlying grid of 1 by 1 is laid over the sampling region. Then an ideal value of window.dims would be (2,3) since its entries evenly divide the width (20) and height (30), respectively, of the sampling region. Using the vector (3, 4.5) would imply that some data will not be used in the moving windows since these values would create partial blocks in the sampling region.
 #'
-#'To preserve the spatial dependence structure, the spatial blocks should have the same shape (i.e. square or rectangle) and orientation as the entire sampling domain.
+#'The value window.dims provides the width and height of the moving window in terms of the underlying grid laid on the sampling region. For example, if a grid with dimensions of grid.spacing = c(0.1, 0.2) is laid on the sampling region and window.dims = c(2,3) then the dimensions of the subblocks created by the moving windows are (0.2, 0.6). Thus, the user must take care to ensure that the values of grid.spacing and window.dims are compatible with the dimensions of the sampling region.
+#'
+#'To preserve the spatial dependence structure, the moving window should have the same shape (i.e. square or rectangle) and orientation as the entire sampling domain.
 #'
 #' @return \item{gamma.hat}{A matrix of the spatial lags provided and the semivariogram point estimates at those lags used to construct the test statistic.}
 #' \item{sigma.hat}{The estimate of asymptotic variance-covariance matrix, Sigma, used to construct test statistic.} 
@@ -29,7 +31,6 @@
 #' \item{test.stat}{The calculated test statistic.}
 #' \item{pvalue.finite}{The approximate, finite-sample adjusted p-value computed by using the moving windows (see Guan et. al. (2004), Section 3.3 for details).}
 #' \item{pvalue.chisq}{The p-value computed using the asymptotic Chi-sq distribution.}
-#' \item{h.sb}{The value of h (bandwidth) used to estimate the semivariogram on the moving windows (blocks).}
 #'
 #' @references Guan, Y., Sherman, M., & Calvin, J. A. (2004). A nonparametric test for spatial isotropy using subsampling. \emph{Journal of the American Statistical Association}, 99(467), 810-821.
 #'
@@ -56,13 +57,14 @@
 #' mydata <- cbind(coords, z)
 #' mylags = rbind(c(1,0), c(0, 1), c(1, 1), c(-1,1))
 #' myA = rbind(c(1, -1, 0 , 0), c(0, 0, 1, -1))
-#' mysb.dims = c(4,4)
+#' my.grid = c(1,1)
+#' my.windims = c(4,4)
 #' myh = 0.7
 #' myh.sb = 0.8
 #' my.xlims = c(0, 20)
 #' my.ylims = c(0, 20)
-#' tr <- GuanTestUnif(mydata, mylags, myA, df = 2, mysb.dims, myh, "norm", 1.5,
-#'  my.xlims, my.ylims, 1, myh.sb)
+#' tr <- GuanTestUnif(mydata, mylags, myA, df = 2, myh, "norm", 1.5,
+#'  my.xlims, my.ylims, my.grid,my.windims, myh.sb)
 #' tr
 #'
 #' #Simulate data from anisotropic covariance function
@@ -77,55 +79,65 @@
 #' z <- t(z)
 #' mydata <- cbind(coords, z)
 #' #Run the test on the data generated from an anisotropic covariance function
-#' tr <- GuanTestUnif(mydata, mylags, myA, df = 2, mysb.dims, myh, "norm", 1.5, 
-#' my.xlims, my.ylims, 1, myh.sb)
+#' tr <- GuanTestUnif(mydata, mylags, myA, df = 2, myh, "norm", 1.5,
+#'  my.xlims, my.ylims, my.grid,my.windims, myh.sb)
 #' tr
 
-GuanTestUnif = function(spdata, lagmat, A, df, subblock.dims, h = 1, kernel = "norm", truncation = 1.5, xlims, ylims, grid.spacing = 1, subblock.h, sig.est.finite = T)
+GuanTestUnif = function(spdata, lagmat, A, df, h = 1, kernel = "norm", truncation = 1.5, xlims, ylims, grid.spacing = c(1,1), window.dims = c(2,2), subblock.h, sig.est.finite = T)
 {
 	if(dim(spdata)[2] != 3)
-	{stop("matrix of spatial data must have 3 columns")}
+	{stop("matrix of spatial data (spdata) must have 3 columns")}
 	if(dim(spdata)[1] <= 3)
-	{stop("matrix of spatial data must have at least 4 rows")}
+	{stop("matrix of spatial data (spdata) must have at least 4 rows")}
 	if(dim(lagmat)[2] != 2)
-	{stop("matrix of spatial lags must have 2 columns")}
+	{stop("matrix of spatial lags (lagmat) must have 2 columns")}
 	if(dim(lagmat)[1] != dim(A)[2])
 	{stop("non-conformable `A` and `lagmat` matrix")}
 	if(df <= 0)
 	{stop("df must be greater than 0")}
-	if(length(subblock.dims) != 2)
-	{stop("subblock.dims must be length 2")}
-	if(subblock.dims[1] <= 0 | subblock.dims[2] <= 0)
-	{stop("subblock dimensions must be positive")}
-	if(xlims[1] >= xlims[2])
-	{stop("invalid x limits of sampling region")}
-	if(ylims[1] >= ylims[2])
-	{stop("invalid y limits of sampling region")}
-	if(df <= 0)
-	{stop("df must be greater than 0")}
-	if(grid.spacing <= 0)
-	{stop("grid.spacing value must be greater than 0")}
+	if(h <= 0)
+	{stop("bandwidth h must be positve")}
 	if(kernel != "norm" & kernel != "ep" & kernel != "cos" & kernel != "unif")
 	{stop("invalid kernel name entered")}
 	if(truncation <= 0)
 	{stop("trunction parameter must be positive")}
-	if(h <= 0)
-	{stop("bandwidth must be positve")}
+	if(length(xlims) != 2 | length(ylims) != 2)
+	{stop("invalid x or y limits")}
+	if(xlims[1] >= xlims[2])
+	{stop("invalid x limits of sampling region")}
+	if(ylims[1] >= ylims[2])
+	{stop("invalid y limits of sampling region")}
+	if(length(grid.spacing) != 2)
+	{stop("grid.spacing must be length 2")}
+	if(grid.spacing[1] <= 0 | grid.spacing[2] <= 0)
+	{stop("grid.spacing values must be greater than 0")}
+	region.w <- xlims[2]-xlims[1]
+	region.h <- ylims[2]-ylims[1]
+	if(region.w %% grid.spacing[1]  != 0)
+	{stop("grid.spacing[1] must evenly divide width of sampling region")}
+	if(region.h %% grid.spacing[2]  != 0)
+	{stop("grid.spacing[2] must evenly divide height of sampling region")}
+	if(length(window.dims) != 2)
+	{stop("window.dims must be length 2")}
+	if(window.dims[1] <= 0 | window.dims[2] <= 0)
+	{stop("window.dims must be positive")}
+	if( (window.dims[1] %% 1) != 0 | (window.dims[2] %% 1) != 0)
+	{warning("window.dims should be integer values")}
 	if(subblock.h <= 0)
-	{stop("subblock bandwidth must be positive")}
+	{stop("subblock.h must be positive")}
 	if(subblock.h > 1)
-	{warning("Recommend subblock.h to be less than 1 to maintain nominal size")}
-	
-	
-	if( (xlims[2]-xlims[1]) <= subblock.dims[1])
-	{stop("subblock width must be less than the width of sampling region")}
-	if( (ylims[2]-ylims[1]) <= subblock.dims[2])
-	{stop("subblock height must be less than the height of sampling region")}
-
-	if( ((xlims[2]-xlims[1])%%subblock.dims[1]) != 0 )
-	{warning("width of subblock does not divide width of sampling region evenly, some data will be ommited during subsampling (function does not handle incomplete subbocks)")}
-	if( ((ylims[2]-ylims[1])%%subblock.dims[2]) != 0 )
-	{warning("height of subblock does not divide height of sampling region evenly, some data will be ommited during subsampling (function does not handle incomplete subbocks)")}
+	{warning("recommend subblock.h to be less than 1 to maintain nominal size")}
+	#Check window dimensions
+	win.w <- window.dims[1]*grid.spacing[1]
+	win.h <- window.dims[2]*grid.spacing[2]
+	if( (xlims[2]-xlims[1]) <= win.w)
+	{stop("window width must be less than the width of sampling region")}
+	if( (ylims[2]-ylims[1]) <= win.h)
+	{stop("window height must be less than the height of sampling region")}
+	if( ((xlims[2]-xlims[1])%%win.w) != 0 )
+	{warning("width of windows does not divide width of sampling region evenly, some data will be ommited during subsampling (function does not handle incomplete subbocks)")}
+	if( ((ylims[2]-ylims[1])%%win.h) != 0 )
+	{warning("height of windows does not divide height of sampling region evenly, some data will be ommited during subsampling (function does not handle incomplete subbocks)")}
 	
 	n.lags <- dim(lagmat)[1]
 	
@@ -133,7 +145,7 @@ GuanTestUnif = function(spdata, lagmat, A, df, subblock.dims, h = 1, kernel = "n
 	gamma.hat <- est_gamma3(rawdata, lagmat, h, kernel, truncation)
 	gh <- gamma.hat[,3]
 	
-	sig.data <- est_subblock_irreg(spdata, lagmat, xlims, ylims, subblock.dims[1], subblock.dims[2], grid.spacing, subblock.h, kernel, truncation)
+	sig.data <- est_subblock_irreg(spdata, lagmat, xlims, ylims, window.dims, grid.spacing, subblock.h, kernel, truncation)
 	
 	block.ghats <- sig.data$gamma.hats
 	ghat.mean <-  apply(block.ghats, 2, mean)
@@ -152,7 +164,7 @@ GuanTestUnif = function(spdata, lagmat, A, df, subblock.dims, h = 1, kernel = "n
 	if(sig.est.finite == T)
 	{
 		vol.Dn <- (xlims[2]-xlims[1])*(ylims[2]-ylims[1])
-		vol.Dni <- subblock.dims[1]*subblock.dims[2]
+		vol.Dni <- win.w*win.h
 		fn <- 1 - (vol.Dni/vol.Dn)
 		kn <-  sig.data$n.good.blks
 		sigma.hat <- ( vol.Dni/(kn*fn) )*t(block.ghats.mc) %*% block.ghats.mc
@@ -172,7 +184,7 @@ GuanTestUnif = function(spdata, lagmat, A, df, subblock.dims, h = 1, kernel = "n
 		
 	blk.sizes <- sig.data$blk.sizes
 	n.blks <- sig.data$n.good.blks
-	blk.vol <- subblock.dims[1]*subblock.dims[2]
+	blk.vol <- win.w*win.h
 	block.test.stats <-  c()
 	for(i in 1:n.blks)
 	{
@@ -190,5 +202,5 @@ GuanTestUnif = function(spdata, lagmat, A, df, subblock.dims, h = 1, kernel = "n
 
 	pvalue.chisq <- pchisq(test.stat, df, lower.tail = F)
 	
-	rv <- list("gamma.hat" = gamma.hat, "sigma.hat" = sigma.hat, "n.subblocks" = n.blks, "test.stat" = test.stat,"pvalue.finite" = pvalue.finite, "pvalue.chisq" = pvalue.chisq, "h.sb" = subblock.h)
+	rv <- list("gamma.hat" = gamma.hat, "sigma.hat" = sigma.hat, "n.subblocks" = n.blks, "test.stat" = test.stat,"pvalue.finite" = pvalue.finite, "pvalue.chisq" = pvalue.chisq)
 }
