@@ -9,16 +9,20 @@
 #' @param lagmat A \eqn{k} by \eqn{2} matrix of spatial lags. Each row corresponds to a lag of the form \eqn{(x.lag, y.lag)} for which the covariogram value will be estimated.
 #' @param A	A \eqn{d} by \eqn{k} contrast matrix. The contrasts correspond to contrasts of the estimated covariogram at the lags given in 'lagmat'.
 #' @param df A scalar indicating the row rank of A. This value gives the degrees of freedom for the asymptotic Chi-sq distribution used to compute the p-value.
-#' @param subblock.dims	A vector of length two corresponding to the width and height of the blocks used in the GBBB. If block width does not evenly divide the width of the sampling region, some data will be ommited during subsampling, i.e., function does not handle partial blocks. Same applies to block height and height of sampling region.
 #' @param xlims A vector of length two providing the lower and upper x-limits of the sampling region.
 #' @param ylims A vector of length two providing the lower and upper y-limits of the sampling region.
+#' @param grid.spacing A vector of length two providing the x (width) and y (height) spacing, respectively, of the underlying grid laid on the sampling region to create spatial blocks. If the grid spacing width does not evenly divide the width of the sampling region, some data will be ommited during subsampling, i.e., the function does not handle partial windows. Same applies to grid spacing height and height of sampling region. See details for an example.
+#' @param block.dims A vector of length two corresponding to the width and height of the blocks used in the GBBB. The dimensions are given in terms of the spacing of the grid laid on the sampling region. See details for an example.
 #' @param nBoot A scalar indicating the number of grid based block bootstrap (GBBB) samples to compute (see Lahiri and Zhu (2006) for details on GBBB).
-#' @param grid A vector of length two indicating the width and height of the underlying grid laid on the sampling region to create spatial blocks.
 #' @param kappa A scalar corresponding to the tuning parameter to adjust empirical bandwidth.
 #' @param user.bandwidth Logical. Set to true to manually override the empirical bandwidth.
 #' @param bandwidth A vector of length two providing the user-definted bandwidths for smoothing over spatial lags in the x and y directions. Only used when user.bandwidth = TRUE.
 #'
-#' @details This function currently only supports square and rectangular sampling regions and does not currently support partial blocks. For example, suppose the sampling region runs from 0 to 20 in the x-direction and from 0 to 30 in the y-direction and an underlying grid of 1 by 1 is laid over the sampling region. Then an ideal value of subblock.dims would be (2,3) since its entries evenly divide the width (20) and height (30), respectively, of the sampling region. Using the vector (3, 4.5) would imply that some data will not be used in the GBBB since these values would create partial blocks in the sampling region. To preserve the spatial dependence structure, the spatial blocks should have the same shape (i.e. square or rectangle) and orientation as the entire sampling domain.
+#' @details This function currently only supports square and rectangular sampling regions and does not support partial blocks. For example, suppose the sampling region runs from 0 to 20 in the x-direction and from 0 to 30 in the y-direction and an underlying grid of 1 by 1 is laid over the sampling region. Then an ideal value of block.dims would be (2,3) since  this would imply the blocks created have dimension 2 by 3 and these values evenly divide the width (20) and height (30), respectively, of the sampling region. Using the vector (3, 4.5) would imply that some data will not be used in the GBBB since these values would create partial blocks in the sampling region. 
+#'
+#'The value block.dims provides the width and height of the blocks in terms of the underlying grid laid on the sampling region. For example, if a grid with dimensions of grid.spacing = c(0.1, 0.2) is laid on the sampling region and block.dims = c(2,3) then the dimensions of the blocks created by the moving windows are (0.2, 0.6). The block dimensions of 0.2 and 0.6 should evenly divide the width and height, respectively, of the entire sampling region.
+#'
+#'To preserve the spatial dependence structure, the spatial blocks should have the same shape (i.e. square or rectangle) and orientation as the entire sampling domain.
 #'
 #' @return \item{C.hat}{A matrix of the spatial lags provided and the covariogram point estimates at those lags used to construct the test statistic.}
 #' \item{V.hat}{The estimate of the asymptotic variance-covariance matrix, V, used to construct the test statistic.}
@@ -52,14 +56,15 @@
 #' mydata <- cbind(coords, z)
 #' mylags <- rbind(c(1,0), c(0, 1), c(1, 1), c(-1,1))
 #' myA <- rbind(c(1, -1, 0 , 0), c(0, 0, 1, -1))
-#' mysb.dims <- c(4,4)
-#' my.xlims <- c(0, 20)
-#' my.ylims <- c(0, 20)
+#' my.xlims <- c(0, 16)
+#' my.ylims <- c(0, 16)
+#' my.grid <- c(1,1)
+#' my.blockdims <- c(4,4)
 #' #Number of bootstraps for demonstration only.
 #' #In practice, use nBoot > 50
 #' my.nBoot <- 3
-#' tr <- MaityTest(mydata, mylags, myA, df = 2, mysb.dims, my.xlims, my.ylims, 
-#' nBoot = my.nBoot, grid = c(1,1))
+#' tr <- MaityTest(mydata, mylags, myA, df = 2, my.xlims, my.ylims, my.grid, 
+#' my.blockdims, nBoot = my.nBoot)
 #' tr
 #'
 #' ####NOT RUN####
@@ -75,10 +80,10 @@
 #' # z <- t(z)
 #' # mydata <- cbind(coords, z)
 #' # #Run the test on the data generated from an anisotropic covariance function
-#' # tr <- MaityTest(mydata, mylags, myA, df = 2, mysb.dims, my.xlims, my.ylims, 
-#' # nBoot = my.nBoot, grid = c(1,1))
+#' # tr <- MaityTest(mydata, mylags, myA, df = 2, my.xlims, my.ylims, 
+#' # my.grid, my.blockdims, nBoot = my.nBoot)
 #' # tr
-MaityTest = function(spdata, lagmat, A, df, subblock.dims, xlims, ylims, nBoot = 100, grid = c(1,1), kappa = 1, user.bandwidth = F, bandwidth = c(1,1))
+MaityTest = function(spdata, lagmat, A, df, xlims, ylims, grid.spacing = c(1,1), block.dims, nBoot = 100, kappa = 1, user.bandwidth = F, bandwidth = c(1,1))
 {
 	if(dim(spdata)[2] != 3)
 	{stop("matrix of spatial data must have 3 columns")}
@@ -94,36 +99,30 @@ MaityTest = function(spdata, lagmat, A, df, subblock.dims, xlims, ylims, nBoot =
 	{stop("invalid x limits of sampling region")}
 	if(ylims[1] >= ylims[2])
 	{stop("invalid y limits of sampling region")}
+	if(length(block.dims) != 2)
+	{stop("block.dims must be length 2")}
+	if(block.dims[1] <= 0 | block.dims[2] <= 0)
+	{stop("block.dims must have positive entries")}
+	if( (block.dims[1] %% 1) != 0 | (block.dims[2] %% 1) != 0)
+	{warning("block.dims should be integer values")}
+	block.w <- grid.spacing[1]*block.dims[1]
+	block.h <- grid.spacing[2]*block.dims[2]
+	if( (xlims[2]-xlims[1]) <= block.w)
+	{stop("block width must be less than the width of sampling region, check grid.spacing[1] and block.dims[1]")}
+	if( (ylims[2]-ylims[1]) <= block.h)
+	{stop("block height must be less than the height of sampling region, check grid.spacing[2] and block.dims[2]")}
+	if( ((xlims[2]-xlims[1])%%block.w) != 0 )
+	{warning("width of blocks does not divide width of sampling region evenly, some data will be ommited during subsampling (function does not handle incomplete subbocks)")}
+	if( ((ylims[2]-ylims[1])%%block.h) != 0 )
+	{warning("height of blocks does not divide height of sampling region evenly, some data will be ommited during subsampling (function does not handle incomplete subbocks)")}
 	if(nBoot <= 0)
 	{stop("invalid number of bootstraps")}
-	if(length(subblock.dims) != 2)
-	{stop("subblock.dims must be length 2")}
-	if(subblock.dims[1] <= 0 | subblock.dims[2] <= 0)
-	{stop("subblock dimensions must be positive")}
-	if(kappa <= 0)
-	{stop("invalid value of kappa")}
-	if(subblock.dims[1] < grid[1])
-	{stop("blk.dim[1] must be >= grid[1]")}
-	if(subblock.dims[2] < grid[2])
-	{stop("blk.dim[2] must be >= grid[2]")}
-	if(user.bandwidth == T & length(bandwidth != 2))
-	{stop("user provided bandwidth must have length 2")}
-	
-	if(grid[1] > subblock.dims[1])
-	{stop("subblock width must be >= underlying grid width")}
-	if(grid[2] > subblock.dims[2])
-	{stop("subblock height must be >= underlying grid height")}	
-	if( (xlims[2]-xlims[1]) <= subblock.dims[1])
-	{stop("subblock width must be less than the width of sampling region")}
-	if( (ylims[2]-ylims[1]) <= subblock.dims[2])
-	{stop("subblock height must be less than the height of sampling region")}
-
-	if( ((xlims[2]-xlims[1])%%subblock.dims[1]) != 0 )
-	{warning("width of subblock does not divide width of sampling region evenly, some data will be ommited during subsampling (function does not handle incomplete subbocks)")}
-	if( ((ylims[2]-ylims[1])%%subblock.dims[2]) != 0 )
-	{warning("height of subblock does not divide height of sampling region evenly, some data will be ommited during subsampling (function does not handle incomplete subbocks)")}
 	if(nBoot < 50)
 	{warning("at least 50 bootstrap samples are recommended")}
+	if(kappa <= 0)
+	{stop("kappa must be positive")}
+	if(user.bandwidth == T & length(bandwidth != 2))
+	{stop("bandwidth must have length 2")}
 
 	rawdata <- lag_dist_prod(spdata)
 	bad <- which(rawdata[,1] == 0 & rawdata[,2] == 0)
@@ -131,7 +130,7 @@ MaityTest = function(spdata, lagmat, A, df, subblock.dims, xlims, ylims, nBoot =
 	chat.mat <- est_chat_MS(rawdata, lagmat, kappa = 1, user.bandwidth, bandwidth)
 	chat <- chat.mat[,3]
 	
-	blk.chats <- est_block_chats(lagmat, spdata, nBoot, subblock.dims, xlims, ylims, grid, kappa, user.bandwidth, bandwidth)
+	blk.chats <- est_block_chats(lagmat, spdata, nBoot, block.dims, xlims, ylims, grid.spacing, kappa, user.bandwidth, bandwidth)
 	Vhat <- cov(blk.chats)
 	Tn <-  t(A %*% chat) %*% solve(A %*% Vhat %*% t(A)) %*% (A %*% chat)
 	Tn <- c(Tn)
